//
//  AppDelegate.m
//  MotoCharger
//
//  Created by Scott Gruby on 11/23/06.
//  Copyright 2006 Scott Gruby. All rights reserved.
//

#import "AppDelegate.h"

//================================================================================================
//
//	DeviceAdded
//
//	This routine is the callback for our IOServiceAddMatchingNotification.  When we get called
//	we will look at all the devices that were added and we will:
//
//	1.  Create some private data to relate to each device (in this case we use the service's name
//	    and the location ID of the device
//	2.  Submit an IOServiceAddInterestNotification of type kIOGeneralInterest for this device,
//	    using the refCon field to store a pointer to our private data.  When we get called with
//	    this interest notification, we can grab the refCon and access our private data.
//
//================================================================================================
void DeviceAdded(void *refCon, io_iterator_t iterator)
{
    kern_return_t		kr;
    io_service_t		usbDevice;
    IOCFPlugInInterface	**plugInInterface = NULL;
    SInt32				score;
    HRESULT 			res;
	AppDelegate			*controller = (AppDelegate *) refCon;
    
    while ((usbDevice = IOIteratorNext(iterator)))
	{
        OSStatus		result;
		UInt16			vendorID;
		UInt16			productID;
		IOUSBDeviceInterface	**deviceInterface;
		
		result = [controller IORegistryGetUInt16:usbDevice property:CFSTR("idVendor") number:&vendorID];
    	if (noErr == result)
    	{
			result = [controller IORegistryGetUInt16:usbDevice property:CFSTR("idProduct") number:&productID];
    	}
		

		if (noErr != result || ![controller hasMatchingID:vendorID inProduct:productID])
        {
			continue;
		}
	   
        // Now, get the locationID of this device. In order to do this, we need to create an IOUSBDeviceInterface 
        // for our device. This will create the necessary connections between our userland application and the 
        // kernel object for the USB Device.
        kr = IOCreatePlugInInterfaceForService(usbDevice, kIOUSBDeviceUserClientTypeID, kIOCFPlugInInterfaceID,
                                               &plugInInterface, &score);
		
        if ((kIOReturnSuccess != kr) || !plugInInterface)
		{
            continue;
        }
		
        // Use the plugin interface to retrieve the device interface.
        res = (*plugInInterface)->QueryInterface(plugInInterface, CFUUIDGetUUIDBytes(kIOUSBDeviceInterfaceID),
                                                 (LPVOID*) &deviceInterface);
        
        // Now done with the plugin interface.
        (*plugInInterface)->Release(plugInInterface);
		
        if (res || deviceInterface == NULL)
		{
            continue;
        }
		
		kr = (*deviceInterface)->USBDeviceOpen(deviceInterface);
		
		if (KERN_SUCCESS == kr)
		{
			kr = (*deviceInterface)->SetConfiguration(deviceInterface, 1);
			
			if (KERN_SUCCESS != kr)
			{
				continue;
			}
			kr = (*deviceInterface)->USBDeviceClose(deviceInterface);
		}
		
		kr = (*deviceInterface)->Release(deviceInterface);
		
        // Done with this USB device; release the reference added by IOIteratorNext
        kr = IOObjectRelease(usbDevice);
    }
}

@implementation AppDelegate
- (void) awakeFromNib
{
	CFMutableDictionaryRef 	matchingDict;
    CFRunLoopSourceRef		runLoopSource;
	kern_return_t			kr;
    
	mMatchDictionary = [[NSDictionary alloc] initWithContentsOfFile:[[NSBundle mainBundle] pathForResource:@"Devices" ofType:@"plist"]];

	// Set up the matching criteria for the devices we're interested in. The matching criteria needs to follow
    // the same rules as kernel drivers: mainly it needs to follow the USB Common Class Specification, pp. 6-7.
    // See also Technical Q&A QA1076 "Tips on USB driver matching on Mac OS X" 
	// <http://developer.apple.com/qa/qa2001/qa1076.html>.
    // One exception is that you can use the matching dictionary "as is", i.e. without adding any matching 
    // criteria to it and it will match every IOUSBDevice in the system. IOServiceAddMatchingNotification will 
    // consume this dictionary reference, so there is no need to release it later on.
    
    matchingDict = IOServiceMatching(kIOUSBDeviceClassName);	// Interested in instances of class
                                                                // IOUSBDevice and its subclasses
    if (matchingDict == NULL)
	{
		NSLog(@"matchingDict == NULL");
		[NSApp terminate:self];
        return;
    }
    
    // We are interested in all USB devices (as opposed to USB interfaces).  The Common Class Specification
    // tells us that we need to specify the idVendor, idProduct, and bcdDevice fields, or, if we're not interested
    // in particular bcdDevices, just the idVendor and idProduct.  Note that if we were trying to match an 
    // IOUSBInterface, we would need to set more values in the matching dictionary (e.g. idVendor, idProduct, 
    // bInterfaceNumber and bConfigurationValue.
    
    // Create a notification port and add its run loop event source to our run loop
    // This is how async notifications get set up.
    
    mNotifyPort = IONotificationPortCreate(kIOMasterPortDefault);
    runLoopSource = IONotificationPortGetRunLoopSource(mNotifyPort);
    
    CFRunLoopAddSource([[NSRunLoop currentRunLoop] getCFRunLoop], runLoopSource, kCFRunLoopDefaultMode);
    
    // Now set up a notification to be called when a device is first matched by I/O Kit.
    kr = IOServiceAddMatchingNotification(mNotifyPort,					// notifyPort
                                          kIOFirstMatchNotification,	// notificationType
                                          matchingDict,					// matching
                                          DeviceAdded,					// callback
                                          self,							// refCon
                                          &mAddedIter					// notification
                                          );		
	
    // Iterate once to get already-present devices and arm the notification    
    DeviceAdded(self, mAddedIter);	
}

- (void) dealloc
{
	[mMatchDictionary release];
	[super dealloc];
}

- (BOOL) hasMatchingID:(UInt16) inVendor inProduct:(UInt16) inProduct
{
	NSString *vendorIDString = [NSString stringWithFormat:@"%04d", inVendor];
	NSString *productIDString = [NSString stringWithFormat:@"%04d", inProduct];
	
	if (vendorIDString == nil || productIDString == nil)
	{
		return NO;
	}
	
	NSDictionary  *vendorDict = [mMatchDictionary objectForKey:vendorIDString];
	if (vendorDict)
	{
		NSArray *keys = [vendorDict allKeys];
		if ([keys count] == 0)
		{
			return YES;
		}
		
		if ([vendorDict objectForKey:productIDString] != nil)
		{
			return YES;
		}
		
	}
	
	return NO;
}


- (OSStatus)  IORegistryGetUInt16:(io_service_t) service property:(CFStringRef) property number:(UInt16 *) number
{
	OSStatus	result = noErr;
	CFNumberRef	numberRef = NULL;
	
	numberRef = (CFNumberRef)IORegistryEntryCreateCFProperty(service, property, kCFAllocatorDefault, 0);
    if (NULL == numberRef)
	{
		result = -1;
	}
	else
	{
		if (false == CFNumberGetValue(numberRef, kCFNumberSInt16Type, number))
		{
			result = -1;
		}
		CFRelease(numberRef);
	}
	return result;
}
@end
